@@ -1,7 +1,7 @@
 // qjackctlJackConnect.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -68,81 +68,6 @@ jack_port_t *qjackctlJackPort::jackPort (void)
 }
 
 
-// Special port name sorting virtual comparator.
-int qjackctlJackPort::compare (QListViewItem* pPortItem, int iColumn, bool bAscending) const
-{
-    QString sName1, sName2;
-    QString sPrefix1, sPrefix2;
-    int iSuffix1, iSuffix2;
-    int i, iNumber, iDigits;
-    int iDecade, iFactor;
-
-    sName1 = text(iColumn);
-    sName2 = pPortItem->text(iColumn);
-
-    iNumber  = 0;
-    iDigits  = 0;
-    iDecade  = 1;
-    iFactor  = 1;
-    iSuffix1 = 0;
-    for (i = sName1.length() - 1; i >= 0; i--) {
-        QCharRef ch = sName1.at(i);
-        if (ch.isDigit()) {
-            iNumber += ch.digitValue() * iDecade;
-            iDecade *= 10;
-            iDigits++;
-        } else {
-            sPrefix1.insert(0, ch.lower());
-            if (iDigits > 0) {
-                iSuffix1 += iNumber * iFactor;
-                iFactor *= 100;
-                iNumber  = 0;
-                iDigits  = 0;
-                iDecade  = 1;
-            }
-        }
-    }
-
-    iNumber  = 0;
-    iDigits  = 0;
-    iDecade  = 1;
-    iFactor  = 1;
-    iSuffix2 = 0;
-    for (i = sName2.length() - 1; i >= 0; i--) {
-        QCharRef ch = sName2.at(i);
-        if (ch.isDigit()) {
-            iNumber += ch.digitValue() * iDecade;
-            iDecade *= 10;
-            iDigits++;
-        } else {
-            sPrefix2.insert(0, ch.lower());
-            if (iDigits > 0) {
-                iSuffix2 += iNumber * iFactor;
-                iFactor *= 100;
-                iNumber  = 0;
-                iDigits  = 0;
-                iDecade  = 1;
-            }
-        }
-    }
-
-    if (sPrefix1 == sPrefix2) {
-        if (iSuffix1 < iSuffix2)
-            return (bAscending ? -1 :  1);
-        else
-        if (iSuffix1 > iSuffix2)
-            return (bAscending ?  1 : -1);
-        else
-            return 0;
-    } else {
-        if (sPrefix1 < sPrefix2)
-            return (bAscending ? -1 :  1);
-        else
-            return (bAscending ?  1 : -1);
-    }
-}
-
-
 //----------------------------------------------------------------------
 // class qjackctlJackClient -- Jack client list item.
 //
@@ -167,7 +92,7 @@ qjackctlJackClient::~qjackctlJackClient (void)
 // Jack client accessor.
 jack_client_t *qjackctlJackClient::jackClient (void)
 {
-    return ((qjackctlJackClientList *) clientlist())->jackClient();
+    return ((qjackctlJackClientList *) clientList())->jackClient();
 }
 
 
