@@ -1,7 +1,7 @@
 // qjackctlPatchbay.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2006, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -37,12 +37,6 @@
 #include "qjackctlConnectAlias.h"
 
 
-// Common pixmap resources.
-static int      g_iXpmRefCount = 0;
-// Exclusive socket overlay pixmap.
-static QPixmap *g_pXpmXSocket1 = 0;
-
-
 //----------------------------------------------------------------------
 // class qjackctlConnectToolTip -- custom list view tooltips.
 
@@ -140,7 +134,9 @@ int qjackctlPlugItem::rtti (void) const
 //
 
 // Constructor.
-qjackctlSocketItem::qjackctlSocketItem ( qjackctlSocketList *pSocketList, const QString& sSocketName, const QString& sClientName, int iSocketType, bool bExclusive, qjackctlSocketItem *pSocketAfter )
+qjackctlSocketItem::qjackctlSocketItem ( qjackctlSocketList *pSocketList,
+	const QString& sSocketName, const QString& sClientName,
+	int iSocketType, qjackctlSocketItem *pSocketAfter )
     : QListViewItem(pSocketList->listView(), pSocketAfter)
 {
     QListViewItem::setText(0, sSocketName);
@@ -149,7 +145,8 @@ qjackctlSocketItem::qjackctlSocketItem ( qjackctlSocketList *pSocketList, const
     m_sSocketName = sSocketName;
     m_sClientName = sClientName;
     m_iSocketType = iSocketType;
-    m_bExclusive  = bExclusive;
+    m_bExclusive  = false;
+	m_sSocketForward = QString::null;
 
     m_plugs.setAutoDelete(false);
     m_connects.setAutoDelete(false);
@@ -196,6 +193,12 @@ bool qjackctlSocketItem::isExclusive (void)
     return m_bExclusive;
 }
 
+const QString& qjackctlSocketItem::forward (void)
+{
+    return m_sSocketForward;
+}
+
+
 void qjackctlSocketItem::setSocketName ( const QString& sSocketName )
 {
     m_sSocketName = sSocketName;
@@ -216,6 +219,11 @@ void qjackctlSocketItem::setExclusive ( bool bExclusive )
     m_bExclusive = bExclusive;
 }
 
+void qjackctlSocketItem::setForward ( const QString& sSocketForward )
+{
+    m_sSocketForward = sSocketForward;
+}
+
 
 // Socket flags accessor.
 bool qjackctlSocketItem::isReadable (void)
@@ -318,10 +326,7 @@ void qjackctlSocketItem::updatePixmap (void)
 // Constructor.
 qjackctlSocketList::qjackctlSocketList( qjackctlSocketListView *pListView, bool bReadable )
 {
-    if (g_iXpmRefCount == 0) {
-        g_pXpmXSocket1 = new QPixmap(QPixmap::fromMimeSource("xsocket1.png"));
-    }
-    g_iXpmRefCount++;
+    QPixmap pmXSocket1(QPixmap::fromMimeSource("xsocket1.png"));
 
     m_pListView   = pListView;
     m_bReadable   = bReadable;
@@ -331,18 +336,22 @@ qjackctlSocketList::qjackctlSocketList( qjackctlSocketListView *pListView, bool
     if (bReadable) {
         m_sSocketCaption = tr("Output");
         m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET]   = new QPixmap(QPixmap::fromMimeSource("asocketo.png"));
-        m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET_X] = createPixmapMerge(*m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET], *g_pXpmXSocket1);
+        m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET_X] = createPixmapMerge(*m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET], pmXSocket1);
+		m_apPixmaps[QJACKCTL_XPM_AUDIO_CLIENT]   = new QPixmap(QPixmap::fromMimeSource("acliento.png"));
         m_apPixmaps[QJACKCTL_XPM_AUDIO_PLUG]     = new QPixmap(QPixmap::fromMimeSource("aportlno.png"));
         m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET]    = new QPixmap(QPixmap::fromMimeSource("msocketo.png"));
-        m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET_X]  = createPixmapMerge(*m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET], *g_pXpmXSocket1);
+        m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET_X]  = createPixmapMerge(*m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET], pmXSocket1);
+		m_apPixmaps[QJACKCTL_XPM_MIDI_CLIENT]    = new QPixmap(QPixmap::fromMimeSource("mcliento.png"));
         m_apPixmaps[QJACKCTL_XPM_MIDI_PLUG]      = new QPixmap(QPixmap::fromMimeSource("mporto.png"));
     } else {
         m_sSocketCaption = tr("Input");
         m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET]   = new QPixmap(QPixmap::fromMimeSource("asocketi.png"));
-        m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET_X] = createPixmapMerge(*m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET], *g_pXpmXSocket1);
+        m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET_X] = createPixmapMerge(*m_apPixmaps[QJACKCTL_XPM_AUDIO_SOCKET], pmXSocket1);
+		m_apPixmaps[QJACKCTL_XPM_AUDIO_CLIENT]   = new QPixmap(QPixmap::fromMimeSource("aclienti.png"));
         m_apPixmaps[QJACKCTL_XPM_AUDIO_PLUG]     = new QPixmap(QPixmap::fromMimeSource("aportlni.png"));
         m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET]    = new QPixmap(QPixmap::fromMimeSource("msocketi.png"));
-        m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET_X]  = createPixmapMerge(*m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET], *g_pXpmXSocket1);
+        m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET_X]  = createPixmapMerge(*m_apPixmaps[QJACKCTL_XPM_MIDI_SOCKET], pmXSocket1);
+		m_apPixmaps[QJACKCTL_XPM_MIDI_CLIENT]    = new QPixmap(QPixmap::fromMimeSource("mclienti.png"));
         m_apPixmaps[QJACKCTL_XPM_MIDI_PLUG]      = new QPixmap(QPixmap::fromMimeSource("mporti.png"));
     }
 
@@ -360,13 +369,6 @@ qjackctlSocketList::~qjackctlSocketList (void)
 
     for (int iPixmap = 0; iPixmap < QJACKCTL_XPM_PIXMAPS; iPixmap++)
         delete m_apPixmaps[iPixmap];
-
-    if (g_iXpmRefCount > 0) {
-        if (--g_iXpmRefCount == 0) {
-            delete g_pXpmXSocket1;
-            g_pXpmXSocket1 = 0;
-        }
-    }
 }
 
 
@@ -490,7 +492,7 @@ void qjackctlSocketList::clientPortsSnapshot (void)
                     if (pSocket == 0) {
                         pSocket = new qjackctlSocketItem(this, sClientName,
 							qjackctlClientAlias::escapeRegExpDigits(sClientName),
-							QJACKCTL_SOCKETTYPE_AUDIO, false,
+							QJACKCTL_SOCKETTYPE_AUDIO,
 							(qjackctlSocketItem *) m_pListView->lastItem());
 					}
                     if (pSocket && pPlug == 0) {
@@ -541,7 +543,7 @@ void qjackctlSocketList::clientPortsSnapshot (void)
                         if (pSocket == 0) {
                             pSocket = new qjackctlSocketItem(this, sClientName,
 								qjackctlClientAlias::escapeRegExpDigits(sClientName),
-								QJACKCTL_SOCKETTYPE_MIDI, false,
+								QJACKCTL_SOCKETTYPE_MIDI,
 								(qjackctlSocketItem *) m_pListView->lastItem());
 						}
                         if (pSocket && pPlug == 0) {
@@ -590,7 +592,7 @@ bool qjackctlSocketList::addSocketItem (void)
 		pSocketForm->setCaption("<" + tr("New") + "> - " + m_sSocketCaption);
         pSocketForm->setSocketCaption(m_sSocketCaption);
         pSocketForm->setPixmaps(m_apPixmaps);
-        pSocketForm->setReadable(m_bReadable);
+		pSocketForm->setSocketList(this);
         pSocketForm->setJackClient(m_pJackClient);
         pSocketForm->setAlsaSeq(m_pAlsaSeq);
         qjackctlPatchbaySocket socket(m_sSocketCaption + " " + QString::number(m_sockets.count() + 1), QString::null, QJACKCTL_SOCKETTYPE_AUDIO);
@@ -601,8 +603,10 @@ bool qjackctlSocketList::addSocketItem (void)
         //  m_pListView->setUpdatesEnabled(false);
             if (pSocketItem)
                 pSocketItem->setSelected(false);
-            pSocketItem = new qjackctlSocketItem(this, socket.name(), socket.clientName(), socket.type(), socket.isExclusive(), pSocketItem);
+            pSocketItem = new qjackctlSocketItem(this, socket.name(), socket.clientName(), socket.type(), pSocketItem);
             if (pSocketItem) {
+				pSocketItem->setExclusive(socket.isExclusive());
+				pSocketItem->setForward(socket.forward());
                 qjackctlPlugItem *pPlugItem = NULL;
                 for (QStringList::Iterator iter = socket.pluglist().begin(); iter != socket.pluglist().end(); iter++)
                     pPlugItem = new qjackctlPlugItem(pSocketItem, *iter, pPlugItem);
@@ -657,13 +661,13 @@ bool qjackctlSocketList::editSocketItem (void)
 				+ " - " + m_sSocketCaption);
             pSocketForm->setSocketCaption(m_sSocketCaption);
             pSocketForm->setPixmaps(m_apPixmaps);
-            pSocketForm->setReadable(m_bReadable);
+			pSocketForm->setSocketList(this);
             pSocketForm->setJackClient(m_pJackClient);
             pSocketForm->setAlsaSeq(m_pAlsaSeq);
             pSocketForm->setConnectCount(pSocketItem->connects().count());
             qjackctlPatchbaySocket socket(pSocketItem->socketName(), pSocketItem->clientName(), pSocketItem->socketType());
-            if (pSocketItem->isExclusive())
-                socket.setExclusive(true);
+			socket.setExclusive(pSocketItem->isExclusive());
+			socket.setForward(pSocketItem->forward());
             for (qjackctlPlugItem *pPlugItem = pSocketItem->plugs().first(); pPlugItem; pPlugItem = pSocketItem->plugs().next())
                 socket.pluglist().append(pPlugItem->plugName());
             pSocketForm->load(&socket);
@@ -676,6 +680,7 @@ bool qjackctlSocketList::editSocketItem (void)
                 pSocketItem->setClientName(socket.clientName());
                 pSocketItem->setSocketType(socket.type());
                 pSocketItem->setExclusive(socket.isExclusive());
+                pSocketItem->setForward(socket.forward());
                 pSocketItem->updatePixmap();
                 qjackctlPlugItem *pPlugItem = NULL;
                 for (QStringList::Iterator iter = socket.pluglist().begin(); iter != socket.pluglist().end(); iter++)
@@ -716,7 +721,7 @@ bool qjackctlSocketList::copySocketItem (void)
 				+ " <" + tr("Copy") + "> - " + m_sSocketCaption);
             pSocketForm->setSocketCaption(m_sSocketCaption);
             pSocketForm->setPixmaps(m_apPixmaps);
-            pSocketForm->setReadable(m_bReadable);
+			pSocketForm->setSocketList(this);
             pSocketForm->setJackClient(m_pJackClient);
             pSocketForm->setAlsaSeq(m_pAlsaSeq);
 			qjackctlPatchbaySocket socket(sSocketName, pSocketItem->clientName(), pSocketItem->socketType());
@@ -725,10 +730,12 @@ bool qjackctlSocketList::copySocketItem (void)
             for (qjackctlPlugItem *pPlugItem = pSocketItem->plugs().first(); pPlugItem; pPlugItem = pSocketItem->plugs().next())
                 socket.pluglist().append(pPlugItem->plugName());
 			pSocketForm->load(&socket);
-			if (pSocketForm->exec()) {	        
+			if (pSocketForm->exec()) {
 				pSocketForm->save(&socket);
-				pSocketItem = new qjackctlSocketItem(this, socket.name(), socket.clientName(), socket.type(), socket.isExclusive(), pSocketItem);
+				pSocketItem = new qjackctlSocketItem(this, socket.name(), socket.clientName(), socket.type(), pSocketItem);
 				if (pSocketItem) {
+					pSocketItem->setExclusive(socket.isExclusive());
+					pSocketItem->setForward(socket.forward());
 					qjackctlPlugItem *pPlugItem = NULL;
 					for (QStringList::Iterator iter = socket.pluglist().begin(); iter != socket.pluglist().end(); iter++)
 						pPlugItem = new qjackctlPlugItem(pSocketItem, *iter, pPlugItem);
@@ -1043,6 +1050,21 @@ qjackctlPatchworkView::~qjackctlPatchworkView (void)
 {
 }
 
+
+// Legal socket item position helper.
+int qjackctlPatchworkView::itemY ( QListViewItem *pItem ) const
+{
+	QListViewItem *pParent = pItem->parent();
+	if (pParent == 0 || pParent->isOpen()) {
+		return (pItem->listView())->itemPos(pItem)
+			+ pItem->height() / 2 - (pItem->listView())->contentsY();
+	} else {
+		return (pParent->listView())->itemPos(pParent)
+			+ pParent->height() / 2 - (pParent->listView())->contentsY();
+	}
+}
+
+
 // Draw visible socket connection relation lines
 void qjackctlPatchworkView::drawConnectionLine ( QPainter& p, int x1, int y1, int x2, int y2, int h1, int h2 )
 {
@@ -1071,6 +1093,33 @@ void qjackctlPatchworkView::drawConnectionLine ( QPainter& p, int x1, int y1, in
 }
 
 
+// Draw socket forwrading line (for input sockets / right pane only)
+void qjackctlPatchworkView::drawForwardLine ( QPainter& p, int x, int dx, int y1, int y2, int h )
+{
+	// Account for list view headers.
+	y1 += h;
+	y2 += h;
+	dx += 4;
+
+	// Draw it...
+	if (y1 < y2) {
+		p.drawLine(x - dx, y1 + 4, x, y1);
+		p.drawLine(x - dx, y1 + 4, x - dx, y2 - 4);
+		p.drawLine(x - dx, y2 - 4, x, y2);
+		// Down arrow...
+		p.drawLine(x - dx, y2 - 8, x - dx - 2, y2 - 12);
+		p.drawLine(x - dx, y2 - 8, x - dx + 2, y2 - 12);
+	} else {
+		p.drawLine(x - dx, y1 - 4, x, y1);
+		p.drawLine(x - dx, y1 - 4, x - dx, y2 + 4);
+		p.drawLine(x - dx, y2 + 4, x, y2);
+		// Up arrow...
+		p.drawLine(x - dx, y2 + 8, x - dx - 2, y2 + 12);
+		p.drawLine(x - dx, y2 + 8, x - dx + 2, y2 + 12);
+	}
+}
+
+
 // Draw visible socket connection relation arrows.
 void qjackctlPatchworkView::drawConnections (void)
 {
@@ -1078,53 +1127,64 @@ void qjackctlPatchworkView::drawConnections (void)
         return;
 
     QPainter p(this);
-    int   x1, y1, h1;
-    int   x2, y2, h2;
-    int   i, c, rgb[3];
+    int x1, y1, h1;
+    int x2, y2, h2;
+	int i, rgb[3] = { 0x99, 0x66, 0x33 };
 
     // Initialize color changer.
-    i = c = rgb[0] = rgb[1] = rgb[2] = 0;
+	i = 0;
     // Almost constants.
     x1 = 0;
     x2 = width();
     h1 = ((m_pPatchbayView->OListView())->header())->sectionRect(0).height();
     h2 = ((m_pPatchbayView->IListView())->header())->sectionRect(0).height();
     // For each client item...
-    for (qjackctlSocketItem *pOSocket = m_pPatchbayView->OSocketList()->sockets().first();
-            pOSocket;
-                pOSocket = m_pPatchbayView->OSocketList()->sockets().next()) {
+	qjackctlSocketItem *pOSocket, *pISocket;
+	QPtrList<qjackctlSocketItem>& osockets
+		= m_pPatchbayView->OSocketList()->sockets();
+	for (pOSocket = osockets.first(); pOSocket; pOSocket = osockets.next()) {
         // Set new connector color.
-        c += 0x39;
-        c &= 0xff;
-        rgb[i++] = c;
-        p.setPen(QColor(rgb[2], rgb[1], rgb[0]));
-        if (i > 2)
-            i = 0;
+		++i; p.setPen(QColor(rgb[i % 3], rgb[(i / 3) % 3], rgb[(i / 9) % 3]));
         // Get starting connector arrow coordinates.
-        QListViewItem* pOParent = pOSocket->parent();
-        if (pOParent == 0 || pOParent->isOpen()) {
-            y1 = (m_pPatchbayView->OListView())->itemPos(pOSocket) + 
-                 pOSocket->height() / 2 - (m_pPatchbayView->OListView())->contentsY();
-        } else {
-            y1 = (m_pPatchbayView->OListView())->itemPos(pOParent) + 
-                 pOParent->height() / 2 - (m_pPatchbayView->OListView())->contentsY();
-        }
+		y1 = itemY(pOSocket);
         // Get input socket connections...
-        for (qjackctlSocketItem *pISocket = pOSocket->connects().first();
-                pISocket;
-                    pISocket = pOSocket->connects().next()) {
+        for (pISocket = pOSocket->connects().first();
+                pISocket; pISocket = pOSocket->connects().next()) {
             // Obviously, there is a connection from pOPlug to pIPlug items:
-            QListViewItem* pIParent = pISocket->parent();
-            if (pIParent == 0 || pIParent->isOpen()) {
-                y2 = (m_pPatchbayView->IListView())->itemPos(pISocket) + 
-                      pISocket->height() / 2 - (m_pPatchbayView->IListView())->contentsY();
-            } else {
-                y2 = (m_pPatchbayView->IListView())->itemPos(pIParent) + 
-                      pIParent->height() / 2 - (m_pPatchbayView->IListView())->contentsY();
-            }
+			y2 = itemY(pISocket);
             drawConnectionLine(p, x1, y1, x2, y2, h1, h2);
         }
     }
+
+	// Look for forwarded inputs...
+	QPtrList<qjackctlSocketItem> iforwards;
+	iforwards.setAutoDelete(false);
+	QPtrList<qjackctlSocketItem>& isockets
+		= m_pPatchbayView->ISocketList()->sockets();
+	// Make a local copy of just the forwarding socket list, if any...
+	for (pISocket = isockets.first(); pISocket; pISocket = isockets.next()) {
+		// Check if its forwarded...
+		if (pISocket->forward().isEmpty())
+		    continue;
+	    iforwards.append(pISocket);
+	}
+    // (Re)initialize color changer.
+    i = 0;
+	// Now traverse those for proper connection drawing...
+	int dx = 0;
+	for (pISocket = iforwards.first(); pISocket; pISocket = iforwards.next()) {
+		qjackctlSocketItem *pISocketForward
+			= m_pPatchbayView->ISocketList()->findSocket(pISocket->forward());
+		if (pISocketForward == 0)
+			continue;
+		// Set new connector color.
+		++i; p.setPen(QColor(rgb[i % 3], rgb[(i / 3) % 3], rgb[(i / 9) % 3]));
+		// Get starting connector arrow coordinates.
+		y1 = itemY(pISocketForward);
+		y2 = itemY(pISocket);
+		drawForwardLine(p, x2, dx, y1, y2, h2);
+		dx += 2;
+	}
 }
 
 
@@ -1178,6 +1238,8 @@ qjackctlPatchbayView::qjackctlPatchbayView ( QWidget *pParent, const char *pszNa
 
     m_bBezierLines = false;
 
+	m_pForwardMenu = NULL;
+
     QObject::connect(m_pOListView, SIGNAL(currentChanged(QListViewItem *)), m_pPatchworkView, SLOT(listViewChanged(QListViewItem *)));
     QObject::connect(m_pOListView, SIGNAL(expanded(QListViewItem *)),  m_pPatchworkView, SLOT(listViewChanged(QListViewItem *)));
     QObject::connect(m_pOListView, SIGNAL(collapsed(QListViewItem *)), m_pPatchworkView, SLOT(listViewChanged(QListViewItem *)));
@@ -1229,7 +1291,61 @@ void qjackctlPatchbayView::contextMenu ( const QPoint& pos, qjackctlSocketList *
         pContextMenu->insertSeparator();
         iItemID = pContextMenu->insertItem(tr("Exclusive"), pSocketList, SLOT(exclusiveSocketItem()));
         pContextMenu->setItemChecked(iItemID, bEnabled && pSocketItem->isExclusive());
-        pContextMenu->setItemEnabled(iItemID, bEnabled && (pSocketItem->connects().count() < 2));
+		pContextMenu->setItemEnabled(iItemID, bEnabled /* && (pSocketItem->connects().count() < 2) */);
+		// Construct the forwarding menu,
+		// overriding the last one, if any...
+		if (m_pForwardMenu)
+			delete m_pForwardMenu;
+		m_pForwardMenu = new QPopupMenu(this);
+		// Assume sockets iteration follows item index order (0,1,2...)
+		// and remember that we only do this for input sockets...
+		int iIndex = 0;
+		if (pSocketItem && pSocketList == ISocketList()) {
+			QPtrList<qjackctlSocketItem>& isockets = ISocketList()->sockets();
+			for (qjackctlSocketItem *pISocket = isockets.first();
+					pISocket; pISocket = isockets.next()) {
+				// Must be of same type of target one...
+				int iSocketType = pISocket->socketType();
+				if (iSocketType != pSocketItem->socketType())
+					continue;
+				const QString& sSocketName = pISocket->socketName();
+				if (pSocketItem->socketName() == sSocketName)
+					continue;
+				int iPixmap = 0;
+				switch (iSocketType) {
+				case QJACKCTL_SOCKETTYPE_AUDIO:
+					iPixmap = (pISocket->isExclusive()
+						? QJACKCTL_XPM_AUDIO_SOCKET_X
+						: QJACKCTL_XPM_AUDIO_SOCKET);
+					break;
+				case QJACKCTL_SOCKETTYPE_MIDI:
+					iPixmap = (pISocket->isExclusive()
+						? QJACKCTL_XPM_MIDI_SOCKET_X
+						: QJACKCTL_XPM_MIDI_SOCKET);
+					break;
+				}
+				iItemID = m_pForwardMenu->insertItem(
+					ISocketList()->pixmap(iPixmap), sSocketName);
+				m_pForwardMenu->setItemChecked(iItemID,
+					pSocketItem->forward() == sSocketName);
+				m_pForwardMenu->setItemParameter(iItemID, iIndex);
+				iIndex++;
+			}
+			// Null forward always present,
+			// and has invalid index parameter (-1)...
+			if (iIndex > 0)
+				m_pForwardMenu->insertSeparator();
+			iItemID = m_pForwardMenu->insertItem(tr("(None)"));
+			m_pForwardMenu->setItemChecked(iItemID,
+				pSocketItem->forward().isEmpty());
+			m_pForwardMenu->setItemParameter(iItemID, -1);
+			// We have something here...
+			QObject::connect(m_pForwardMenu, SIGNAL(activated(int)),
+				this, SLOT(activateForwardMenu(int)));
+		}
+		// Add forward menu to the main context menu...
+		iItemID = pContextMenu->insertItem(tr("Forward"), m_pForwardMenu);
+		pContextMenu->setItemEnabled(iItemID, iIndex > 0);
         pContextMenu->insertSeparator();
         iItemID = pContextMenu->insertItem(QIconSet(QPixmap::fromMimeSource("up1.png")),
 			tr("Move Up"), pSocketList, SLOT(moveUpSocketItem()));
@@ -1257,6 +1373,47 @@ void qjackctlPatchbayView::contextMenu ( const QPoint& pos, qjackctlSocketList *
     pContextMenu->exec(pos);
 
     delete pContextMenu;
+
+	delete m_pForwardMenu;
+	m_pForwardMenu = 0;
+}
+
+
+// Select the forwarding socket name from context menu.
+void qjackctlPatchbayView::activateForwardMenu ( int iItemID )
+{
+	if (m_pForwardMenu == NULL)
+		return;
+
+	int iIndex = m_pForwardMenu->itemParameter(iItemID);
+
+	// Get currently input socket (assume its nicely selected)
+	qjackctlSocketItem *pSocketItem = ISocketList()->selectedSocketItem();
+	if (pSocketItem) {
+		// Check first for forward from nil...
+		if (iIndex < 0) {
+			pSocketItem->setForward(QString::null);
+			setDirty(true);
+			return;
+		}
+		// Hopefully, its a real socket about to be forwraded...
+		QPtrList<qjackctlSocketItem>& isockets = ISocketList()->sockets();
+		for (qjackctlSocketItem *pISocket = isockets.first();
+				pISocket; pISocket = isockets.next()) {
+			// Must be of same type of target one...
+			if (pISocket->socketType() != pSocketItem->socketType())
+				continue;
+			const QString& sSocketName = pISocket->socketName();
+			if (pSocketItem->socketName() == sSocketName)
+				continue;
+			if (iIndex == 0) {
+				pSocketItem->setForward(sSocketName);
+				setDirty(true);
+				break;
+			}
+			iIndex--;
+		}
+	}
 }
 
 
@@ -1643,8 +1800,10 @@ void qjackctlPatchbay::loadRackSockets ( qjackctlSocketList *pSocketList, QPtrLi
     pSocketList->clear();
     qjackctlSocketItem *pSocketItem = NULL;
     for (qjackctlPatchbaySocket *pSocket = socketlist.first(); pSocket; pSocket = socketlist.next()) {
-        pSocketItem = new qjackctlSocketItem(pSocketList, pSocket->name(), pSocket->clientName(), pSocket->type(), pSocket->isExclusive(), pSocketItem);
+        pSocketItem = new qjackctlSocketItem(pSocketList, pSocket->name(), pSocket->clientName(), pSocket->type(), pSocketItem);
         if (pSocketItem) {
+			pSocketItem->setExclusive(pSocket->isExclusive());
+			pSocketItem->setForward(pSocket->forward());
             qjackctlPlugItem *pPlugItem = NULL;
             for (QStringList::Iterator iter = pSocket->pluglist().begin(); iter != pSocket->pluglist().end(); iter++)
                 pPlugItem = new qjackctlPlugItem(pSocketItem, *iter, pPlugItem);
@@ -1699,8 +1858,8 @@ void qjackctlPatchbay::saveRackSockets ( qjackctlSocketList *pSocketList, QPtrLi
     for (qjackctlSocketItem *pSocketItem = (qjackctlSocketItem *) pListView->firstChild(); pSocketItem; pSocketItem = (qjackctlSocketItem *) pSocketItem->nextSibling()) {
         qjackctlPatchbaySocket *pSocket = new qjackctlPatchbaySocket(pSocketItem->socketName(), pSocketItem->clientName(), pSocketItem->socketType());
         if (pSocket) {
-            if (pSocketItem->isExclusive())
-                pSocket->setExclusive(true);
+			pSocket->setExclusive(pSocketItem->isExclusive());
+			pSocket->setForward(pSocketItem->forward());
             for (qjackctlPlugItem *pPlugItem = pSocketItem->plugs().first(); pPlugItem; pPlugItem = pSocketItem->plugs().next())
                 pSocket->pluglist().append(pPlugItem->plugName());
             socketlist.append(pSocket);
@@ -1731,8 +1890,8 @@ void qjackctlPatchbay::saveRack ( qjackctlPatchbayRack *pPatchbayRack )
             qjackctlPatchbaySocket *pISocket = pPatchbayRack->findSocket(pPatchbayRack->isocketlist(), pISocketItem->socketName());
             if (pOSocket && pISocket)
                 pPatchbayRack->addCable(new qjackctlPatchbayCable(pOSocket, pISocket));             
-        }        
-    }            
+        }
+    }
 
     // Reset dirty flag.
     m_pPatchbayView->setDirty(false);
