@@ -1,7 +1,7 @@
 // qjackctlPatchbay.h
 //
 /****************************************************************************
-   Copyright (C) 2003-2005, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2006, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -51,11 +51,13 @@ class qjackctlPatchbay;
 // Pixmap-set indexes.
 #define QJACKCTL_XPM_AUDIO_SOCKET   0
 #define QJACKCTL_XPM_AUDIO_SOCKET_X 1
-#define QJACKCTL_XPM_AUDIO_PLUG     2
-#define QJACKCTL_XPM_MIDI_SOCKET    3
-#define QJACKCTL_XPM_MIDI_SOCKET_X  4
-#define QJACKCTL_XPM_MIDI_PLUG      5
-#define QJACKCTL_XPM_PIXMAPS        6
+#define QJACKCTL_XPM_AUDIO_CLIENT   2
+#define QJACKCTL_XPM_AUDIO_PLUG     3
+#define QJACKCTL_XPM_MIDI_SOCKET    4
+#define QJACKCTL_XPM_MIDI_SOCKET_X  5
+#define QJACKCTL_XPM_MIDI_CLIENT    6
+#define QJACKCTL_XPM_MIDI_PLUG      7
+#define QJACKCTL_XPM_PIXMAPS        8
 
 
 // Custom tooltip class.
@@ -114,7 +116,10 @@ class qjackctlSocketItem : public QListViewItem
 public:
 
     // Constructor.
-    qjackctlSocketItem(qjackctlSocketList *pSocketList, const QString& sSocketName, const QString& sClientName, int iSocketType, bool bExclusive, qjackctlSocketItem *pSocketAfter);
+    qjackctlSocketItem(qjackctlSocketList *pSocketList,
+		const QString& sSocketName, const QString& sClientName,
+		int iSocketType,qjackctlSocketItem *pSocketAfter);
+
     // Default destructor.
     ~qjackctlSocketItem();
 
@@ -123,22 +128,24 @@ public:
     const QString& clientName();
     int socketType();
     bool isExclusive();
+    const QString& forward();
 
     void setSocketName (const QString& sSocketName);
     void setClientName (const QString& sClientName);
     void setSocketType (int iSocketType);
     void setExclusive  (bool bExclusive);
+    void setForward    (const QString& sSocketForward);
 
     // Socket flags accessor.
     bool isReadable();
-    
+
     // Connected plug list primitives.
     void addConnect(qjackctlSocketItem *pSocket);
     void removeConnect(qjackctlSocketItem *pSocket);
 
     // Connected plug  finders.
     qjackctlSocketItem *findConnectPtr(qjackctlSocketItem *pSocketPtr);
-    
+
     // Connection list accessor.
     QPtrList<qjackctlSocketItem>& connects();
 
@@ -160,7 +167,7 @@ public:
 
     // Retrieve a context pixmap.
     QPixmap& pixmap(int iPixmap);
-    
+
     // Update pixmap to its proper context.
     void updatePixmap();
 
@@ -172,6 +179,7 @@ private:
     QString m_sClientName;
     int m_iSocketType;
     bool m_bExclusive;
+    QString m_sSocketForward;
 
     // Plug (port) list.
     QPtrList<qjackctlPlugItem> m_plugs;
@@ -347,13 +355,17 @@ protected:
     virtual void resizeEvent(QResizeEvent *);
     // Context menu request event handler.
     virtual void contextMenuEvent(QContextMenuEvent *);
-    
+
 private:
 
+	// Legal socket item position helper.
+	int itemY(QListViewItem *pItem) const;
+
     // Drawing methods.
+	void drawForwardLine(QPainter& p, int x, int dx, int y1, int y2, int h);
     void drawConnectionLine(QPainter& p, int x1, int y1, int x2, int y2, int h1, int h2);
     void drawConnections();
-    
+
     // Local instance variables.
     qjackctlPatchbayView *m_pPatchbayView;
 };
@@ -381,7 +393,7 @@ public:
     // Patchbay object binding methods.
     void setBinding(qjackctlPatchbay *pPatchbay);
     qjackctlPatchbay *binding();
-    
+
     // Socket list accessors.
     qjackctlSocketList *OSocketList();
     qjackctlSocketList *ISocketList();
@@ -401,9 +413,10 @@ signals:
 
 public slots:
 
-    // Common context menu slot.
+    // Common context menu slots.
     void contextMenu(const QPoint& pos, qjackctlSocketList *pSocketList);
-    
+    void activateForwardMenu(int iItemID);
+
 private:
 
     // Child controls.
@@ -413,10 +426,13 @@ private:
 
     // The main binding object.
     qjackctlPatchbay *m_pPatchbay;
-    
+
     // How we'll draw patchwork lines.
     bool m_bBezierLines;
 
+    // Working forward popup menu.
+    QPopupMenu *m_pForwardMenu;
+
     // The obnoxious dirty flag.
     bool m_bDirty;
 };
